@@ -21,8 +21,8 @@ abess <- function(x, ...) UseMethod("abess")
 #' by the \code{survival} package (recommended) or
 #' a two-column matrix with columns named \code{"time"} and \code{"status"}.
 #' For \code{family = "mgaussian"}, \code{y} should be a matrix of quantitative responses.
-#' For \code{family = "multinomial"}, \code{y} should be a factor of at least three levels.
-#' Note that, for either \code{"binomial"} or \code{"multinomial"},
+#' For \code{family = "multinomial"} or \code{"ordinal"}, \code{y} should be a factor of at least three levels.
+#' Note that, for either \code{"binomial"}, \code{"ordinal"} or \code{"multinomial"},
 #' if y is presented as a numerical vector, it will be coerced into a factor.
 #' @param family One of the following models:
 #' \code{"gaussian"} (continuous response),
@@ -31,6 +31,7 @@ abess <- function(x, ...) UseMethod("abess")
 #' \code{"cox"} (left-censored response),
 #' \code{"mgaussian"} (multivariate continuous response),
 #' \code{"multinomial"} (multi-class response),
+#' \code{"ordinal"} (multi-class ordinal response),
 #' \code{"gamma"} (positive continuous response).
 #' Depending on the response. Any unambiguous substring can be given.
 #' @param tune.path The method to be used to select the optimal support size. For
@@ -45,7 +46,6 @@ abess <- function(x, ...) UseMethod("abess")
 #' @param gs.range A integer vector with two elements.
 #' The first element is the minimum model size considered by golden-section,
 #' the later one is the maximum one. Default is \code{gs.range = c(1, min(n, round(n/(log(log(n))log(p)))))}.
-#' Not available now.
 #' @param lambda A single lambda value for regularized best subset selection. Default is 0.
 #' @param always.include An integer vector containing the indexes of variables that should always be included in the model.
 #' @param group.index A vector of integers indicating the which group each variable is in.
@@ -54,6 +54,8 @@ abess <- function(x, ...) UseMethod("abess")
 #' Denote the first group as \code{1}, the second \code{2}, etc.
 #' If you do not fit a model with a group structure,
 #' please set \code{group.index = NULL} (the default).
+#' @param init.active.set A vector of integers indicating the initial active set. 
+#' Default: \code{init.active.set = NULL}. 
 #' @param splicing.type Optional type for splicing.
 #' If \code{splicing.type = 1}, the number of variables to be spliced is
 #' \code{c.max}, ..., \code{1}; if \code{splicing.type = 2},
@@ -65,13 +67,15 @@ abess <- function(x, ...) UseMethod("abess")
 #' important variables to be splicing.
 #' When \code{important.search} \eqn{\ll} \code{p} variables,
 #' it would greatly reduce runtimes. Default: \code{important.search = 128}.
-#' @param normalize Options for normalization. \code{normalize = 0} for no normalization.
-#' \code{normalize = 1} for subtracting the mean of columns of \code{x}.
-#' \code{normalize = 2} for scaling the columns of \code{x} to have \eqn{\sqrt n} norm.
-#' \code{normalize = 3} for subtracting the means of the columns of \code{x} and \code{y}, and also
+#' @param normalize Options for normalization. 
+#' \code{normalize = 0} for no normalization.
+#' \code{normalize = 1} for subtracting the means of the columns of \code{x} and \code{y}, and also
 #' normalizing the columns of \code{x} to have \eqn{\sqrt n} norm.
-#' If \code{normalize = NULL}, \code{normalize} will be set \code{1} for \code{"gaussian"},
-#' \code{2} for \code{"binomial"}. Default is \code{normalize = NULL}.
+#' \code{normalize = 2} for subtracting the mean of columns of \code{x} and 
+#' scaling the columns of \code{x} to have \eqn{\sqrt n} norm.
+#' \code{normalize = 3} for scaling the columns of \code{x} to have \eqn{\sqrt n} norm.
+#' If \code{normalize = NULL}, \code{normalize} will be set \code{1} for \code{"gaussian"} and \code{"mgaussian"},
+#' \code{3} for \code{"cox"}. Default is \code{normalize = NULL}.
 #' @param c.max an integer splicing size. Default is: \code{c.max = 2}.
 #' @param weight Observation weights. When \code{weight = NULL},
 #' we set \code{weight = 1} for each observation as default.
@@ -250,6 +254,17 @@ abess <- function(x, ...) UseMethod("abess")
 #'   support.size = c(3, 4), type = "response"
 #' )
 #'
+#' ################ Ordinal regression  ################
+#' dataset <- generate.data(n, p, support.size, family = "ordinal", class.num = 4)
+#' abess_fit <- abess(dataset[["x"]], dataset[["y"]],
+#'   family = "ordinal", tune.type = "cv"
+#' )
+#' coef <- coef(abess_fit, support.size = abess_fit[["best.size"]])[[1]]
+#' predict(abess_fit,
+#'   newx = dataset[["x"]][1:10, ],
+#'   support.size = c(3, 4), type = "response"
+#' )
+#'
 #' ########## Best group subset selection #############
 #' dataset <- generate.data(n, p, support.size)
 #' group_index <- rep(1:10, each = 2)
@@ -282,7 +297,7 @@ abess.default <- function(x,
                           y,
                           family = c(
                             "gaussian", "binomial", "poisson", "cox",
-                            "mgaussian", "multinomial", "gamma"
+                            "mgaussian", "multinomial", "gamma","ordinal"
                           ),
                           tune.path = c("sequence", "gsection"),
                           tune.type = c("gic", "ebic", "bic", "aic", "cv"),
@@ -294,6 +309,7 @@ abess.default <- function(x,
                           lambda = 0,
                           always.include = NULL,
                           group.index = NULL,
+                          init.active.set = NULL, 
                           splicing.type = 2,
                           max.splicing.iter = 20,
                           screening.num = NULL,
@@ -310,471 +326,91 @@ abess.default <- function(x,
                           num.threads = 0,
                           seed = 1,
                           ...) {
-  tau <- NULL
-
-  ## check lambda
-  stopifnot(length(lambda) == 1)
-  stopifnot(!anyNA(lambda))
-  stopifnot(all(lambda >= 0))
-  lambda.list <- lambda
-  lambda.min <- 0.001
-  lambda.max <- 100
-  nlambda <- 100
-
   set.seed(seed)
-
-  ## check number of thread:
-  stopifnot(is.numeric(num.threads) & num.threads >= 0)
-  num_threads <- as.integer(num.threads)
-
-  ## check early stop:
-  stopifnot(is.logical(early.stop))
-  early_stop <- early.stop
-
-  ## check warm start:
-  stopifnot(is.logical(warm.start))
-
-  ## check splicing type
-  stopifnot(length(splicing.type) == 1)
-  stopifnot(splicing.type %in% c(1, 2))
-  splicing_type <- 2 - splicing.type
-  splicing_type <- as.integer(splicing_type)
-
-  ## check max splicing iteration
-  stopifnot(is.numeric(max.splicing.iter) & max.splicing.iter >= 1)
-  check_integer_warning(
-    max.splicing.iter,
-    "max.splicing.iter should be an integer value.
-                        It is coerced to as.integer(max.splicing.iter)."
-  )
-  max_splicing_iter <- as.integer(max.splicing.iter)
-
-  ## task type:
   family <- match.arg(family)
-  model_type <- switch(family,
-    "gaussian" = 1,
-    "binomial" = 2,
-    "poisson" = 3,
-    "cox" = 4,
-    "mgaussian" = 5,
-    "multinomial" = 6,
-    "gamma" = 8
-  )
-
-  ## check predictors:
-  stopifnot(class(x)[1] %in% c("data.frame", "matrix", "dgCMatrix"))
-  vn <- colnames(x) ## if x is not a matrix type object, it will return NULL.
-  nvars <- ncol(x)
-  nobs <- nrow(x)
-  if (nvars == 1) {
-    stop("x should have at least two columns!")
-  }
-  if (is.null(vn)) {
-    vn <- paste0("x", 1:nvars)
-  }
-  sparse_X <- ifelse(class(x)[1] %in% c("matrix", "data.frame"), FALSE, TRUE)
-  if (sparse_X) {
-    if (class(x) == "dgCMatrix") {
-      x <- map_dgCMatrix2entry(x)
-    }
-  } else {
-    if (is.data.frame(x)) {
-      x <- as.matrix(x)
-    }
-    if (!is.numeric(x)) {
-      stop("x must be a *numeric* matrix/data.frame!")
-    }
-  }
-  if (anyNA(x) || any(is.infinite(x))) {
-    stop("x has missing value or infinite value!")
-  }
-
-  # check weight
-  if (is.null(weight)) {
-    weight <- rep(1, nobs)
-  }
-  stopifnot(is.vector(weight))
-  if (length(weight) != nobs) {
-    stop("Rows of x must be the same as length of weight!")
-  }
-  stopifnot(all(is.numeric(weight)), all(weight >= 0))
-
-  ## check response:
-  if (anyNA(y)) {
-    stop("y has missing value!")
-  }
-  if (any(is.infinite(y))) {
-    stop("y has infinite value!")
-  }
-  if (family == "gaussian") {
-    if (is.matrix(y)) {
-      if (dim(y)[2] > 1) {
-        stop("The dimension of y should not exceed 1 when family = 'gaussian'!")
-      }
-    }
-  }
-  if (family == "binomial" || family == "multinomial") {
-    if (length(unique(y)) == 2 && family == "multinomial") {
-      warning("y is a binary variable and is not match to family = 'multinomial'.
-              We change to family = 'binomial'")
-      model_type <- 2
-      family <- "binomial"
-    }
-    if (length(unique(y)) > 2 && family == "binomial") {
-      stop("Input binary y when family = 'binomial'; otherwise,
-           change the option for family to 'multinomial'. ")
-    }
-    if (length(unique(y)) == nobs && family == "multinomial") {
-      stop("All of y value are distinct.
-           Please input categorial y when family = 'multinomial'.")
-    }
-    if ((nobs / length(unique(y))) < 5 && family == "multinomial") {
-      warning("The number of the category of y is relative large compare to nvars.
-              The numerical result might be unstable.")
-    }
-    if (!is.factor(y)) {
-      y <- as.factor(y)
-    }
-    class.name <- levels(y)
-    y_vn <- class.name
-
-    if (family == "binomial") {
-      y <- as.numeric(y) - 1
-    }
-    if (family == "multinomial") {
-      y <- model.matrix(~ factor(as.numeric(y) - 1) + 0)
-      colnames(y) <- NULL
-    }
-  }
-  if (family == "poisson") {
-    if (any(y < 0)) {
-      stop("y must be positive integer value when family = 'poisson'.")
-    }
-  }
-  if (family == "gamma") {
-    if (any(y < 0)) {
-      stop("y must be positive value when family = 'gamma'.")
-    }
-  }
-  if (family == "cox") {
-    if (!is.matrix(y)) {
-      y <- as.matrix(y)
-    }
-    if (ncol(y) != 2) {
-      stop("y must be a Surv object or a matrix with two columns when family = 'cox'!")
-    }
-    stopifnot(length(unique(y[, 2])) == 2)
-    ## pre-process data for cox model
-    sort_y <- order(y[, 1])
-    y <- y[sort_y, ]
-    x <- x[sort_y, ]
-    y <- y[, 2]
-  }
-  if (family == "mgaussian") {
-    if (!is.matrix(y) || dim(y)[2] <= 1) {
-      stop("y must be a n-by-q matrix (q > 1) when family = 'mgaussian'!")
-    }
-    y_vn <- colnames(y)
-    if (is.null(y_vn)) {
-      y_vn <- colnames("y", 1:dim(y)[2])
-    }
-  }
-  y <- as.matrix(y)
-  y_dim <- ncol(y)
-  multi_y <- family %in% MULTIVARIATE_RESPONSE
-
-  # check whether x and y are matching:
-  if (nobs != nrow(y)) {
-    stop("Rows of x must be the same as rows of y!")
-  }
-
-  ## strategy for tuning
   tune.path <- match.arg(tune.path)
-  if (tune.path == "gsection") {
-    path_type <- 2
-  } else if (tune.path == "sequence") {
-    path_type <- 1
-  }
-
-  ## group variable:
-  group_select <- FALSE
-  if (is.null(group.index)) {
-    g_index <- 1:nvars - 1
-    ngroup <- 1
-    max_group_size <- 1
-    # g_df <- rep(1, nvars)
-  } else {
-    stopifnot(all(!is.na(group.index)))
-    stopifnot(all(is.finite(group.index)))
-    stopifnot(diff(group.index) >= 0)
-    check_integer(group.index, "group.index must be a vector with integer value.")
-    group_select <- TRUE
-    gi <- unique(group.index)
-    g_index <- match(gi, group.index) - 1
-    g_df <- c(diff(g_index), nvars - max(g_index))
-    ngroup <- length(g_index)
-    max_group_size <- max(g_df)
-  }
-
-  # sparse level list (sequence):
-  if (is.null(support.size)) {
-    if (group_select) {
-      s_list <- 0:min(c(ngroup, round(nobs / max_group_size / log(ngroup))))
-    } else {
-      s_list <- 0:min(c(nvars, round(nobs / log(log(nobs)) / log(nvars))))
-    }
-  } else {
-    stopifnot(any(is.numeric(support.size) & support.size >= 0))
-    check_integer(support.size, "support.size must be a vector with integer value.")
-    if (group_select) {
-      stopifnot(max(support.size) <= ngroup)
-    } else {
-      stopifnot(max(support.size) <= nvars)
-    }
-    stopifnot(max(support.size) < nobs)
-    support.size <- sort(support.size)
-    support.size <- unique(support.size)
-    s_list <- support.size
-    # if (s_list[1] == 0) {
-    #   zero_size <- TRUE
-    # } else {
-    #   zero_size <- FALSE
-    #   s_list <- c(0, s_list)
-    # }
-  }
-
-  # sparse range (golden-section):
-  if (is.null(gs.range)) {
-    s_min <- 1
-    if (group_select) {
-      s_max <- min(c(ngroup, round(nobs / max_group_size / log(ngroup))))
-    } else {
-      s_max <- min(c(nvars, round(nobs / log(log(nobs)) / log(nvars))))
-    }
-  } else {
-    stopifnot(length(gs.range) == 2)
-    stopifnot(all(is.numeric(gs.range)))
-    stopifnot(all(gs.range > 0))
-    check_integer_warning(
-      gs.range,
-      "gs.range should be a vector with integer.
-                          It is coerced to as.integer(gs.range)."
-    )
-    gs.range <- as.integer(gs.range)
-    stopifnot(as.integer(gs.range)[1] != as.integer(gs.range)[2])
-    if (group_select) {
-      stopifnot(max(gs.range) < ngroup)
-    } else {
-      stopifnot(max(gs.range) < nvars)
-    }
-    gs.range <- as.integer(gs.range)
-    s_min <- min(gs.range)
-    s_max <- max(gs.range)
-  }
-
-  ## check C-max:
-  stopifnot(is.numeric(c.max))
-  stopifnot(c.max >= 1)
-  check_integer_warning(
-    c.max,
-    "c.max should be an integer. It is coerced to as.integer(c.max)."
-  )
-  c_max <- as.integer(c.max)
-
-  ## check compatible between group selection and support size
-  # if (group_select) {
-  #   if (path_type == 1 & max(s_list) > length(gi))
-  #     stop("The maximum one support.size should not be larger than the number of groups!")
-  #   if (path_type == 2 & s_max > length(gi))
-  #     stop("max(gs.range) is too large. Should be smaller than the number of groups!")
-  # }
-
-  ## check covariance update
-  stopifnot(is.logical(cov.update))
-  if (model_type == 1) {
-    covariance_update <- cov.update
-  } else {
-    covariance_update <- FALSE
-  }
-
-  ## check parameters for sub-optimization:
-  # 1:
-  if (length(newton) == 2) {
-    if (family %in% c("binomial", "cox", "multinomial", "gamma", "poisson")) {
-      newton <- "approx"
-    }
-  }
-  newton <- match.arg(newton)
-  # if (newton == "auto") {
-  #   if (family == "cox") {
-  #     newton <- "approx"
-  #   } else if (family == "logistic") {
-  #     newton <- "auto"
-  #   }
-  # }
-  if (family %in% c("gaussian", "mgaussian")) {
-    newton <- "exact"
-  }
-  newton_type <- switch(newton,
-    "exact" = 0,
-    "approx" = 1,
-    "auto" = 2
-  )
-  approximate_newton <- ifelse(newton_type == 1, TRUE, FALSE)
-  # 2:
-  if (!is.null(max.newton.iter)) {
-    stopifnot(is.numeric(max.newton.iter) & max.newton.iter >= 1)
-    max_newton_iter <- as.integer(max.newton.iter)
-  } else {
-    max_newton_iter <- ifelse(newton_type == 0, 10, 60)
-    if (family == "gamma" && newton_type == 1) {
-      max_newton_iter <- 200
-    }
-  }
-  # 3:
-  stopifnot(is.numeric(newton.thresh) & newton.thresh > 0)
-  newton_thresh <- as.double(newton.thresh)
-
-  # tune support size method:
   tune.type <- match.arg(tune.type)
-  ic_type <- map_tunetype2numeric(tune.type)
-  is_cv <- ifelse(tune.type == "cv", TRUE, FALSE)
-  if (is_cv) {
-    if (is.null(foldid)) {
-      cv_fold_id <- integer(0)
-      nfolds <- check_nfold(nfolds)
-    } else {
-      cv_fold_id <- check_foldid(foldid, nobs)
-      nfolds <- length(unique(nfolds))
-    }
-  } else {
-    cv_fold_id <- integer(0)
-    # nfolds <- 1
-  }
+  
+  data <- list(
+    x = x,
+    y = y
+  )
 
-  ## information criterion
-  stopifnot(is.numeric(ic.scale))
-  stopifnot(ic.scale >= 0)
-  ic_scale <- as.integer(ic.scale)
+  para <- Initialization_GLM(
+    c.max=c.max,
+    support.size=support.size,
+    always.include=always.include,
+    group.index=group.index,
+    splicing.type=splicing.type,
+    max.splicing.iter=max.splicing.iter,
+    warm.start=warm.start,
+    ic.scale=ic.scale,
+    num.threads=num.threads,
+    newton.thresh=newton.thresh,
+    tune.type=tune.type,
+    important.search=important.search,
+    tune.path=tune.path,
+    max.newton.iter=max.newton.iter,
+    lambda=lambda,
+    family=family,
+    screening.num=screening.num,
+    gs.range=gs.range,
+    early.stop=early.stop,
+    weight=weight,
+    cov.update=cov.update,
+    normalize=normalize,
+    init.active.set=init.active.set,
+    newton=newton,
+    foldid=foldid,
+    nfolds=nfolds
+  )
 
-  ## normalize strategy:
-  if (is.null(normalize)) {
-    is_normal <- TRUE
-    normalize <- switch(family,
-      "gaussian" = 1,
-      "binomial" = 2,
-      "poisson" = 2,
-      "cox" = 3,
-      "mgaussian" = 1,
-      "multinomial" = 2,
-      "gamma" = 2
-    )
-  } else {
-    stopifnot(normalize %in% 0:3)
-    if (normalize != 0) {
-      # normalize <- as.character(normalize)
-      # normalize <- switch (normalize,
-      #                      '1' <- 2,
-      #                      '2' <- 3,
-      #                      '3' <- 1
-      # )
-      if (normalize == 1) {
-        normalize <- 2
-      } else if (normalize == 2) {
-        normalize <- 3
-      } else if (normalize == 3) {
-        normalize <- 1
-      } else {
-      }
-      is_normal <- TRUE
-    } else {
-      is_normal <- FALSE
-      normalize <- 0
-    }
-  }
-
-  if (is.null(screening.num)) {
-    screening <- FALSE
-    screening_num <- nvars
-  } else {
-    stopifnot(is.numeric(screening.num))
-    stopifnot(screening.num >= 1)
-    check_integer_warning(
-      screening.num,
-      "screening.num should be a integer.
-                          It is coerced to as.integer(screening.num)."
-    )
-    screening.num <- as.integer(screening.num)
-    if (screening.num > nvars) {
-      stop("The number of screening features must be equal or less than that of the column of x!")
-    }
-    if (path_type == 1) {
-      if (screening.num < max(s_list)) {
-        stop("The number of screening features must be equal or greater than the maximum one in support.size!")
-      }
-    } else {
-      if (screening.num < s_max) {
-        stop("The number of screening features must be equal or greater than the max(gs.range)!")
-      }
-    }
-    screening <- TRUE
-    screening_num <- screening.num
-  }
-
-  # check important searching:
-  if (is.null(important.search)) {
-    important_search <- min(c(nvars, 128))
-    important_search <- as.integer(important_search)
-  } else {
-    stopifnot(is.numeric(important.search))
-    stopifnot(important.search >= 0)
-    check_integer_warning(important.search)
-    important_search <- as.integer(important.search)
-  }
-
-  # check always included variables:
-  if (is.null(always.include)) {
-    always_include <- numeric(0)
-  } else {
-    if (anyNA(always.include) || any(is.infinite(always.include))) {
-      stop("always.include has missing values or infinite values.")
-    }
-    stopifnot(always.include %in% 1:nvars)
-    stopifnot(always.include > 0)
-    check_integer(always.include, "always.include must be a vector with integer value.")
-    always.include <- as.integer(always.include) - 1
-    always_include_num <- length(always.include)
-    if (always_include_num > screening_num) {
-      stop("The number of variables in always.include must not exceed the screening.num")
-    }
-    if (path_type == 1) {
-      if (always_include_num > max(s_list)) {
-        stop("always.include containing too many variables.
-           The length of it must not exceed the maximum in support.size.")
-      }
-      if (always_include_num > min(s_list)) {
-        if (is.null(support.size)) {
-          s_list <- s_list[s_list >= always_include_num]
-        } else {
-          stop(sprintf("always.include containing %s variables. The min(support.size) must be equal or greater than this.", always_include_num))
-        }
-      }
-    } else {
-      if (always_include_num > s_max) {
-        stop("always.include containing too many variables. The length of it must not exceed the max(gs.range).")
-      }
-      if (always_include_num > s_min) {
-        if (is.null(support.size)) {
-          s_min <- always_include_num
-        } else {
-          stop(sprintf("always.include containing %s variables. The min(gs.range) must be equal or greater than this.", always_include_num))
-        }
-      }
-    }
-    always_include <- always.include
-  }
-
-  t1 <- proc.time()
+  model <- initializate(para,data)
+  para <- model$para
+  data <- model$data
+  
+  y <- data$y
+  x <- data$x
+  tune.path <- para$tune.path
+  lambda <- para$lambda
+  family <- para$family
+  gs.range <- para$gs.range
+  weight <- para$weight
+  normalize <- para$normalize
+  init.active.set <- para$init.active.set
+  nfolds <- para$nfolds
+  warm.start <- para$warm.start
+  num_threads  <- para$num_threads 
+  splicing_type  <- para$splicing_type 
+  max_splicing_iter <- para$max_splicing_iter
+  nobs <- para$nobs
+  nvars <- para$nvars
+  vn <- para$vn
+  sparse_X <- para$sparse_X
+  screening_num <- para$screening_num
+  g_index <- para$g_index
+  s_list <- para$s_list
+  c_max <- para$c_max
+  ic_scale <- para$ic_scale
+  important_search <- para$important_search
+  always_include   <- para$always_include  
+  max_newton_iter   <- para$max_newton_iter  
+  path_type <- para$path_type
+  newton_thresh <- para$newton_thresh
+  screening <- para$screening
+  ic_type <- para$ic_type
+  is_cv <- para$is_cv
+  cv_fold_id <- para$cv_fold_id
+  s_min <- para$s_min
+  s_max <- para$s_max
+  model_type <- para$model_type
+  covariance_update <- para$covariance_update
+  approximate_newton <- para$approximate_newton
+  y_vn <- para$y_vn
+  y_dim <- para$y_dim
+  multi_y <- para$multi_y
+  early_stop <- para$early_stop
+  
   result <- abessGLM_API(
     x = x,
     y = y,
@@ -810,24 +446,11 @@ abess.default <- function(x,
     sparse_matrix = sparse_X,
     splicing_type = splicing_type,
     sub_search = important_search,
-    cv_fold_id = cv_fold_id
+    cv_fold_id = cv_fold_id, 
+    A_init = as.integer(init.active.set)
   )
-  t2 <- proc.time()
-  # print(t2 - t1)
 
   ## process result
-
-  ### process best model (abandon):
-  # support.index <- which(result[["beta"]] != 0.0)
-  # names(result[["beta"]]) <- vn
-  # best_model <- list("beta" = result[["beta"]],
-  #                    "coef0" = result[["coef0"]],
-  #                    "support.index" = support.index,
-  #                    "support.size" = sum(result[["beta"]] != 0.0),
-  #                    "dev" = result[["train_loss"]],
-  #                    "tune.value" = result[["ic"]])
-  # result[["best.model"]] <- best_model
-
   result[["beta"]] <- NULL
   result[["coef0"]] <- NULL
   result[["train_loss"]] <- NULL
@@ -838,7 +461,6 @@ abess.default <- function(x,
   result[["nvars"]] <- nvars
   result[["family"]] <- family
   result[["tune.path"]] <- tune.path
-  # result[["support.df"]] <- g_df
   result[["tune.type"]] <- ifelse(is_cv == TRUE, "cv",
     c("AIC", "BIC", "GIC", "EBIC")[ic_type]
   )
@@ -847,9 +469,13 @@ abess.default <- function(x,
   ## preprocessing result in "gsection"
   if (tune.path == "gsection") {
     ## change the order:
-    reserve_order <- length(result[["sequence"]]):1
+    reserve_order <- rev(seq_len(length(result[["sequence"]])))
     result[["beta_all"]] <- result[["beta_all"]][reserve_order]
-    result[["coef0_all"]] <- result[["coef0_all"]][reserve_order, , drop = FALSE]
+    if (is.matrix(result[["coef0_all"]])) {
+      result[["coef0_all"]] <- result[["coef0_all"]][reserve_order, , drop = FALSE]
+    } else {
+      result[["coef0_all"]] <- as.matrix(result[["coef0_all"]][reserve_order])
+    }
     result[["train_loss_all"]] <- result[["train_loss_all"]][reserve_order, , drop = FALSE]
     result[["ic_all"]] <- result[["ic_all"]][reserve_order, , drop = FALSE]
     result[["test_loss_all"]] <- result[["test_loss_all"]][reserve_order, , drop = FALSE]
@@ -883,10 +509,11 @@ abess.default <- function(x,
   }
   result[["tune.value"]] <- result[["tune.value"]][, 1]
 
+  ############ restore intercept ############
   result[["best.size"]] <- s_list[which.min(result[["tune.value"]])]
   names(result)[which(names(result) == "coef0_all")] <- "intercept"
   if (family %in% MULTIVARIATE_RESPONSE) {
-    if (family == "multinomial") {
+    if (family %in% c("multinomial", "ordinal")) {
       result[["intercept"]] <- lapply(result[["intercept"]], function(x) {
         x <- x[-y_dim]
       })
@@ -895,13 +522,11 @@ abess.default <- function(x,
     result[["intercept"]] <- as.vector(result[["intercept"]])
   }
 
+  ############ restore intercept ############
   names(result)[which(names(result) == "beta_all")] <- "beta"
-  # names(result)[which(names(result) == 'screening_A')] <- "screening.index"
-  # result[["screening.index"]] <- result[["screening.index"]] + 1
-
   if (multi_y) {
     if (screening) {
-      for (i in 1:length(result[["beta"]])) {
+      for (i in seq_len(length(result[["beta"]]))) {
         beta_all <- matrix(0, nrow = nvars, ncol = y_dim)
         beta_all[result[["screening_A"]] + 1, ] <- result[["beta"]][[i]]
         result[["beta"]][[i]] <- beta_all
@@ -912,7 +537,7 @@ abess.default <- function(x,
       result[["beta"]] <- lapply(result[["beta"]], Matrix::Matrix,
         sparse = TRUE, dimnames = list(vn, y_vn)
       )
-    } else {
+    } else if (family %in% c("multinomial", "ordinal")) {
       result[["beta"]] <- lapply(result[["beta"]], function(x) {
         Matrix::Matrix(x[, -y_dim], sparse = TRUE, dimnames = list(vn, y_vn[-1]))
       })
@@ -936,30 +561,10 @@ abess.default <- function(x,
   result[["screening.vars"]] <- vn[result[["screening_A"]] + 1]
   result[["screening_A"]] <- NULL
 
-  # if (s_list[0] == 0) {
-  #   nulldev <- result[["dev"]][1]
-  # } else {
-  #   f <- switch(
-  #     family,
-  #     "gaussian" = gaussian(),
-  #     "binomial" = binomial(),
-  #     "poisson" = poisson()
-  #   )
-  #   if (family != "cox") {
-  #     nulldev <- deviance(glm(y ~ .,
-  #                             data = cbind.data.frame(y, 1),
-  #                             family = f))
-  #   } else {
-  #     nulldev <- 0
-  #   }
-  # }
-  # result[["nulldev"]] <- 0
-
   result[["call"]] <- match.call()
   class(result) <- "abess"
 
   set.seed(NULL)
-
   return(result)
 }
 
